/* -
 * Copyright (c) 2002, Ramsey G. Brenner. <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * The contents of this file are subject to the terms of the XORSuite
 * Public Source License (the "License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License
 * at http://rgbrenner.cjb.net/projects/xpsl.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 */

static const char copyright[] =
"@(#) Copyright (c) 2002\n\
  Ramsey G. Brenner <rgbrenner@myrealbox.com>. All rights reserved.\n";

#include "hdrs.h"

char  *cwd;
char  *tmp_dir;
char  *prog_name;

/* Encrypt/Decrypt */
int
main(int argc, char **argv)
{
    tf       fflag;
    int      i;
    char    *tmp,
            *out_name,
            *tmp_name,
           **in_name;
    FILE   **in_fp,
            *out_fp;
    size_t   size,
             nfiles;

    set_general_variables();

    /* Init some variables */
    out_name = NULL;
    size = -1;
    nfiles = 0;
    fflag = FALSE;
    in_name = Malloc(sizeof(char *));
    in_name[0] = NULL;

    /* Init options */
    for(i = 1; i < argc; i++) {
      switch(get_opt_type(argv[i])) {
        case HASH_SHA1:       /* Set hash_algo */
          set_hash(SH);
          break;

        case HASH_MD5:        /* Set hash_algo */
          set_hash(MD);
          break;

        case OUTPUT_FILE:     /* Get output prefix */
          if((out_name = get_opt(argv + i, &i)) == NULL) {
            usage();
            exit(EXIT_FAILURE);
          }
          break;

        case SIZE:            /* Set size of output file */
          if((tmp = get_opt(argv + i, &i)) == NULL) {
            usage();
            exit(EXIT_FAILURE);
          }
          if(strisdigit(tmp)) {
            size = Atoi(tmp);
          } else {
            usage();
            exit(EXIT_FAILURE);
          }
          break;

        case FFLAG:           /* Set fflag */
          fflag = TRUE;
          break;

        case OTHER:           /* Add to file list */
        default:
          nfiles++;
          in_name = Realloc(in_name, (nfiles + 1) * sizeof(char *));
          in_name[nfiles] = NULL;
          in_name[nfiles - 1] = argv[i];
          break;
      }
    }

    /* Check usage */
    if(nfiles < 2) {
      usage();
      exit(EXIT_FAILURE);
    }

    out_name = set_cwd(out_name);

    /* Init in_fp with file pointers */
    /* Allocate memory for file pointers */
    in_fp = Malloc((nfiles + 1) * sizeof(FILE *));
    in_fp[nfiles] = NULL;
    for(i = 0; in_name[i] != NULL; i++) {
      /* Check if file exists */
      if(!chk_file(in_name[i])) {
        Err(MISSING_FILE, in_name[i]);
      }

      /* Check if the hash in the filename
         matches its actual hash */
      chk_hash(in_name[i]);

      /* Open file */
      in_fp[i] = Fopen(in_name[i], "r");
    }

    /* Open temporary FILE */
    out_fp = Tmpfile();

    /* Start Processing */
    /* These #ifdef's arent really needed at this time.
       They are for expansion purposes, but currently
       they are all equivalent. */
#ifdef XORFakeKey
    /* Trim input file to size */
    for(i = 0; in_fp[i] != NULL; i++) {
      trim(&in_fp[i], size);
    }

    /* XOR files */
    XORFiles(in_fp, out_fp);
#endif
#ifdef XOREncrypt
    /* Trim input file to size */
    for(i = 0; in_fp[i] != NULL; i++) {
      trim(&in_fp[i], size);
    }

    /* XOR files */
    XORFiles(in_fp, out_fp);
#endif
#ifdef XORDecrypt
    /* XOR files */
    XORFiles(in_fp, out_fp);

    /* Trim output file to size */
    trim(&out_fp, size);
#endif

    /* Copy out_fp to a tmp fp */
    out_fp = Copy(out_fp, &tmp_name);

    /* Close all files */
    for(i = 0; in_fp[i] != NULL; i++) {
      Fclose(in_fp[i]);
    }
    Fclose(out_fp);

    /* Rename output file */
    Rename(tmp_name, out_name, POSTFIX, fflag);

    exit(EXIT_SUCCESS);
}

void
usage(void)
{
    fprintf(stderr, "XORSuite v%s\n"
      "Usage: %s [options] <input files>\n"
      "Options:\n"
        "\t-f               - Do not prompt if file already exists\n"
        "\t-o <name>        - Output to file <name>\n"
        "\t-s <size>        - Output file will be <size> bytes\n"
        "\t-sha1            - Set hash type to SHA-1 (default)\n"
        "\t-md5             - Set hash type to MD5\n"
        "\n",
      VERSION,
      prog_name);
    return;
}
