/* -
 * Copyright (c) 2002, Ramsey G. Brenner. <rgbrenner@myrealbox.com>
 * All rights reserved.
 *
 * The contents of this file are subject to the terms of the XORSuite
 * Public Source License (the "License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License
 * at http://rgbrenner.cjb.net/projects/xpsl.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 */

static const char copyright[] =
"@(#) Copyright (c) 2002\n\
  Ramsey G. Brenner <rgbrenner@myrealbox.com>. All rights reserved.\n";

#include "hdrs.h"

char  *cwd;
char  *tmp_dir;
char  *prog_name;

/* Create keys */
int
main(int argc, char **argv)
{
    tf       fflag;
    int      i;
    char    *tmp,
            *prefix;
    size_t   nkeys,
             length;

    set_general_variables();

    /* Init some variables */
    nkeys = 1;
    length = -1;
    fflag = FALSE;
    prefix = NULL;

    /* Init options */
    for(i = 1; i < argc; i++) {
      switch(get_opt_type(argv[i])) {
        case HASH_SHA1:     /* Set hash_algo */
          set_hash(SH);
          break;

        case HASH_MD5:      /* Set hash_algo */
          set_hash(MD);
          break;

        case BGEN_SSL:      /* Set bgen_algo */
          set_bgen(SSL);
          break;

        case BGEN_URANDOM:  /* Set bgen_algo */
          set_bgen(URANDOM);
          break;

        case N_OPT:         /* Set number of keys */
          if((tmp = get_opt(argv + i, &i)) == NULL) {
            usage();
            exit(EXIT_FAILURE);
          }
          if(strisdigit(tmp)) {
            nkeys = Atoi(tmp);
          } else {
            usage();
            exit(EXIT_FAILURE);
          }
          break;

        case OUTPUT_FILE:   /* Set output prefix */
          if((prefix = get_opt(argv + i, &i)) == NULL) {
            usage();
            exit(EXIT_FAILURE);
          }
          break;

        case FFLAG:         /* Set fflag */
          fflag = TRUE;
          break;

        case OTHER:         /* Else, must be length */
        default:
          if(length == -1) {
            if(strisdigit(argv[i])) {
              length = Atoi(argv[i]);
            } else {
              usage();
              exit(EXIT_FAILURE);
            }
          } else {
            usage();
            exit(EXIT_FAILURE);
          }
          break;
      }
    }

    /* Check usage */
    if(length == -1) {
      usage();
      exit(EXIT_FAILURE);
    }

    prefix = set_cwd(prefix);

    /* Start key generation */
    for(i = 0; i < nkeys; i++) {
      char  *tmp_name;
      FILE  *key;

      /* Open temporary FILE */
      key = Tmpfile();

      /* Generate key */
      genkey(key, length);

      /* Copy file to final tmp name, and close file pointer */
      Fclose(Copy(key, &tmp_name));

      /* Rename key */
      if(prefix == NULL) {
        Rename(tmp_name, NULL, "key", fflag);
      } else { /* -o <name> was specified */
        char  *out_name;

        /* Assemble output name */
        Asprintf(&out_name, "%s%d", prefix, i + 1);

        Rename(tmp_name, out_name, NULL, fflag);
        Free(out_name);
      }

      /* Cleanup */
      Free(tmp_name);
    }

    exit(EXIT_SUCCESS);
}

void
usage(void)
{
    fprintf(stderr, "XORSuite v%s\n"
      "Usage: %s [options] <length>\n"
      "Options:\n"
        "\t-f               - Do not prompt if file already exists\n"
        "\t-n <number>      - Number of keys to generate (default: 1)\n"
        "\t-o <prefix>      - Set name prefix\n"
        "\t-sha1            - Set hash type to SHA-1 (default)\n"
        "\t-md5             - Set hash type to MD5\n"
        "\t-ssl             - Set byte generator to use SSL (default)\n"
        "\t-urandom         - Set byte generator to use /dev/urandom\n"
        "\n",
      VERSION,
      prog_name);
    return;
}
